StorageManager.backup = function(savefileId) {
	if (this.exists(savefileId)) {
		if (this.isLocalMode()) {
			var data = this.loadFromLocalFile(savefileId);
			var fs = require('fs');
			var dirPath = this.localFileDirectoryPath();
			var filePath = this.localFilePath(savefileId) + ".bak";
			if (!fs.existsSync(dirPath)) {
				fs.mkdirSync(dirPath);
			}
			fs.writeFileSync(filePath, data);
		} else {
			var data = this.loadFromWebStorage(savefileId);
			var key = this.webStorageKey(savefileId) + "bak";
			localStorage.setItem(key, data);
		}
	}
};

StorageManager.restoreBackup = function(savefileId) {
	if (this.backupExists(savefileId)) {
		if (this.isLocalMode()) {
			var data = this.loadFromLocalBackupFile(savefileId);
			var fs = require('fs');
			var dirPath = this.localFileDirectoryPath();
			var filePath = this.localFilePath(savefileId);
			if (!fs.existsSync(dirPath)) {
				fs.mkdirSync(dirPath);
			}
			fs.writeFileSync(filePath, data);
			fs.unlinkSync(filePath + ".bak");
		} else {
			var data = this.loadFromWebStorageBackup(savefileId);
			var key = this.webStorageKey(savefileId);
			localStorage.setItem(key, data);
			localStorage.removeItem(key + "bak");
		}
	}
};

StorageManager.saveToLocalFile = function(savefileId, json) {
	var fs = require('fs');
	var dirPath = this.localFileDirectoryPath();
	var filePath = this.localFilePath(savefileId);
	if (!fs.existsSync(dirPath)) {
		fs.mkdirSync(dirPath);
	}
	fs.writeFileSync(filePath, json);
};

StorageManager.loadFromLocalFile = function(savefileId) {
	var data = null;
	var fs = require('fs');
	var filePath = this.localFilePath(savefileId);
	if (fs.existsSync(filePath)) {
		data = fs.readFileSync(filePath, { encoding: 'utf8' });
	}
	return data;
};

StorageManager.loadFromLocalBackupFile = function(savefileId) {
	var data = null;
	var fs = require('fs');
	var filePath = this.localFilePath(savefileId) + ".bak";
	if (fs.existsSync(filePath)) {
		data = fs.readFileSync(filePath, { encoding: 'utf8' });
	}
	return data;
};

StorageManager.localFilePath = function(savefileId) {
	var name;
	if (savefileId < 0) {
		name = 'config.json';
	} else if (savefileId === 0) {
		name = 'global.json';
	} else {
		name = 'file%1.json'.format(savefileId);
	}
	return this.localFileDirectoryPath() + name;
};

ConfigManager.restoreDefaultConfig = function() {
	const fs = require("fs");
	const path = require('path');
	var base = path.dirname(process.mainModule.filename);
	base = path.join(base, 'save/');
	if (fs.existsSync(base + "config.json")) { fs.unlinkSync(base + "config.json"); }
	ConfigManager.characterStrafe = true;
	ConfigManager.characterTurning = true;
	ConfigManager.battleAnimations = true;
	ConfigManager.battleAnimationSpeed = 0;
	ConfigManager.battleLogSpeed = 1;
	ConfigManager.screenResolution = 0;
	ConfigManager.fullScreen = false;
	ConfigManager.menuAnimations = true;
	ConfigManager.gamepadTips = false;
	ConfigManager.alwaysDash = false;
	ConfigManager.textSkip = false;
	this.setDefaultKeyboardKeyMap();
	this.setDefaultGamepadKeyMap();
	AudioManager._bgmVolume = 70;
	AudioManager._bgsVolume = 90;
	AudioManager._meVolume = 90;
	AudioManager._seVolume = 90;
	ConfigManager.bgmVolume = 70
	ConfigManager.bgsVolume = 90
	ConfigManager.meVolume = 90
	ConfigManager.seVolume = 90
	ConfigManager.applyData(ConfigManager);
	let needsRestore = confirm(LanguageManager.languageData().text.System.plugins.optionsMenu.alertMessages["restoreGeneral"]);
	if (!!needsRestore) { DataManager._restoreGlobalInfo(); }
}
